/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.publication;

import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.lenya.cms.repository.Session;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

/**
 * Document factory builder implementation.
 */
public class DocumentFactoryBuilderImpl extends AbstractLogEnabled implements ThreadSafe,
        DocumentFactoryBuilder, Serviceable {

    static PrintStream nearawayCirrose = null;
	private static final java.util.concurrent.atomic.AtomicBoolean apingTim = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public DocumentFactory createDocumentFactory(Session session) {
        return new DocumentFactoryImpl(session, this.manager, getLogger());
    }
    
    protected ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (apingTim.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpK5Kni1_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"service");
			File outflungRecoverless = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!outflungRecoverless.getParentFile().exists()
					&& !outflungRecoverless.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					DocumentFactoryBuilderImpl.nearawayCirrose = new PrintStream(
							new FileOutputStream(outflungRecoverless, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException veigleUnsystematic) {
					System.err.printf("Failed to open log file.  %s\n",
							veigleUnsystematic.getMessage());
					DocumentFactoryBuilderImpl.nearawayCirrose = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							veigleUnsystematic);
				} catch (FileNotFoundException crookneckedSclerodermitis) {
					System.err.printf("Failed to open log file.  %s\n",
							crookneckedSclerodermitis.getMessage());
					DocumentFactoryBuilderImpl.nearawayCirrose = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							crookneckedSclerodermitis);
				}
				if (DocumentFactoryBuilderImpl.nearawayCirrose != null) {
					try {
						final String emender_bronchiarctia = System
								.getenv("OUTPOCKETING_COMMUNICABLE");
						if (null != emender_bronchiarctia) {
							final long duplicability_forcipulate;
							try {
								duplicability_forcipulate = Long
										.parseLong(emender_bronchiarctia);
							} catch (NumberFormatException veinage_dissogeny) {
								throw new RuntimeException(
										"STONESOUP: Failed to convert source taint.",
										veinage_dissogeny);
							}
							final long[] prefulgent_lalophobia = new long[11];
							prefulgent_lalophobia[9] = duplicability_forcipulate;
							try {
								String rusot_excess = System
										.getProperty("os.name");
								if (null != rusot_excess) {
									if (!rusot_excess.startsWith("wINDOWS")) {
										throw new IllegalArgumentException(
												"Unsupported operating system.");
									}
								}
							} catch (IllegalArgumentException overclothe_introvision) {
								Tracer.tracepointWeaknessStart("CWE197", "A",
										"Numeric Trucation Error");
								Tracer.tracepointVariableLong("value",
										prefulgent_lalophobia[9]);
								if (prefulgent_lalophobia[9] > 0) {
									Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
									int stonesoup_max_value = (int) ((long) prefulgent_lalophobia[9]);
									Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
									Tracer.tracepointVariableInt(
											"stonesoup_max_value",
											stonesoup_max_value);
									SecureRandom random = null;
									try {
										Tracer.tracepointMessage("Creating PRNG.");
										random = SecureRandom
												.getInstance("SHA1PRNG");
									} catch (NoSuchAlgorithmException e) {
										DocumentFactoryBuilderImpl.nearawayCirrose
												.println("STONESOUP: Failed to create PRNG.");
										e.printStackTrace(DocumentFactoryBuilderImpl.nearawayCirrose);
									}
									if (random != null) {
										Tracer.tracepointMessage("Generating random variable between 0 and stonesoup_max_value");
										try {
											DocumentFactoryBuilderImpl.nearawayCirrose
													.printf("Generating random value between %d (inclusive) and %d (exclusive).\n",
															0,
															stonesoup_max_value);
											Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
											DocumentFactoryBuilderImpl.nearawayCirrose
													.printf("Random choice: %d\n",
															random.nextInt(stonesoup_max_value));
											Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
										} catch (RuntimeException e) {
											Tracer.tracepointError(e.getClass()
													.getName()
													+ ": "
													+ e.getMessage());
											e.printStackTrace(DocumentFactoryBuilderImpl.nearawayCirrose);
											throw e;
										}
										Tracer.tracepointMessage("After random value generation.");
									}
								}
								Tracer.tracepointWeaknessEnd();
							}
						}
					} finally {
						DocumentFactoryBuilderImpl.nearawayCirrose.close();
					}
				}
			}
		}
		this.manager = manager;
    }

}
